*******************************************************************************
*   Title: appendix_accountants.do
* Purpose: Produces final tables for appendix.
*          See the main() program at the bottom for organization.
*   Owner: Eric Zwick
*    Date: 2020-04-07
*******************************************************************************

* Table A2
capture program drop table_carryback_stimulus/*%<*/
program define table_carryback_stimulus

    tempfile orig

    use $analysis/clients, clear
    merge 1:1 ein nol_yr using $analysis/sim_refunds, keep(1 3) nogen

    corr sim_refund_13 sim_refund_EZ_13

    gen sim_refund_corrected = sim_refund_EZ
    replace sim_refund_corrected = sim_refund_5yr if nol_yr == 2001 | nol_yr == 2002

    * Split up refunds into buckets based on whether they are larger because of
    * the policy
    
    ****************************************************************************
    * Take-up rates in stimulus years versus non-stimulus years.
    ****************************************************************************
    xi i.nol_yr
    foreach v of varlist sim_refund_13 revenue_13 assets_13 ebitda_13 {
        destring `v', force replace
        gen l`v' = log(`v')
        gen mis_l`v' = l`v' == .
        replace l`v' = -1 if l`v' == .
    }

    gen stimulus_year = inlist(nol_yr, 2001, 2002, 2008, 2009)
    gen stimulus_refund_bonus = log(sim_refund_corrected/sim_refund_2yr)
    gen bigger_refund = stimulus_refund_bonus > 0 & stimulus_refund_bonus < .
    gen stimulus = cond(stimulus_refund_bonus > 0.2 & stimulus_refund_bonus != ., 1, 0)
    save `orig'

    use `orig', clear
    reg iobs_refund stimulus_year, cluster(ein)
    reg iobs_refund stimulus_year lsim_refund_13 lrevenue_13 mis_lrevenue_13 lassets_13 mis_lassets_13 lebitda_13 mis_lebitda_13, cluster(ein)

    ****************************************************************************
    * Are firms more or less likely to take up the carrybacks that are
    *       specifically larger or changed due to a policy?
    ****************************************************************************
    tabstat stimulus_refund_bonus, by(nol_yr) s(mean N)
    tabstat iobs_refund, by(bigger_refund) s(mean N)

    reg iobs_refund bigger_refund, cluster(ein)
    reg iobs_refund bigger_refund lsim_refund_13 lrevenue_13 mis_lrevenue_13 lassets_13 mis_lassets_13 lebitda_13 mis_lebitda_13, cluster(ein)
    reg iobs_refund bigger_refund lsim_refund_13 _Inol_yr*, cluster(ein)
    reg iobs_refund bigger_refund lsim_refund_13 _Inol_yr* lrevenue_13 mis_lrevenue_13 lassets_13 mis_lassets_13 lebitda_13 mis_lebitda_13, cluster(ein)

    reg iobs_refund stimulus, cluster(ein)
    reg iobs_refund stimulus lsim_refund_13 lrevenue_13 mis_lrevenue_13 lassets_13 mis_lassets_13 lebitda_13 mis_lebitda_13, cluster(ein)
    reg iobs_refund stimulus lsim_refund_13 _Inol_yr*, cluster(ein)
    reg iobs_refund stimulus lsim_refund_13 _Inol_yr* lrevenue_13 mis_lrevenue_13 lassets_13 mis_lassets_13 lebitda_13 mis_lebitda_13, cluster(ein)

    reg iobs_refund stimulus_refund_bonus if stimulus_refund_bonus != 0, cluster(ein)
    reg iobs_refund stimulus_refund_bonus lsim_refund_13 lrevenue_13 mis_lrevenue_13 lassets_13 mis_lassets_13 ///
        lebitda_13 mis_lebitda_13 if stimulus_refund_bonus != 0 , cluster(ein)

    ****************************************************************************
    * Identify which firms experience an exogenous increase in carryback
    *     stocks due to the policies,
    *     then regress takeup of carrybacks on preparer characteristics (maybe
    *     focusing on stable preparers).
    ****************************************************************************
    use `orig', clear
    merge m:1 prep_ssn nol_yr using "$analysis/preps", keep(1 3) nogen

    * Focus on stable preparers
    gen external_prep = cond(!regexm(prep_ein, "X.*") | !regexm(prep_ssn, "Y.*"), 1, 0)
    keep if external_prep == 1
    bys ein: gen ein_count = _N
    bys ein prep_ssn: gen ein_prep_count = _N
    gen stable_prep = ein_count == ein_prep_count
    tab stable_prep
    keep if stable_prep == 1

    * Restrict to obs with bigger refunds due to stimulus
    tab bigger_refund

    #delimit ;
    xi: reg iobs_refund bigger_refund##(c.cpa c.atty c.other mis_profession) i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, cluster(ein); 

    xi: reg iobs_refund bigger_refund##(c.ltot_wages_13 mis_ltot_wages_13)
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, cluster(ein); 

    xi: reg iobs_refund bigger_refund##(c.self_emp_ind mis_self_emp_ind)
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, cluster(ein); 

    xi: reg iobs_refund bigger_refund##(c.age mis_age)
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, cluster(ein); 

    xi: reg iobs_refund bigger_refund##(c.lmean_revenue_13 mis_lmean_revenue_13)
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, cluster(ein); 

    xi: reg iobs_refund 
            bigger_refund##(c.cpa c.atty c.other mis_profession
                c.ltot_wages_13 mis_ltot_wages_13
                c.self_emp_ind mis_self_emp_ind
                c.age mis_age
                c.lmean_revenue_13 mis_lmean_revenue_13)
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, cluster(ein); 

    #delimit cr

    keep if bigger_refund == 1

    * Regressions like Table 5 in paper but without fixed effects
    #delimit ;
    xi: reg iobs_refund cpa atty other mis_profession i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, cluster(ein); 

    xi: reg iobs_refund ltot_wages_13 mis_ltot_wages_13
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, cluster(ein); 

    xi: reg iobs_refund self_emp_ind mis_self_emp_ind
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, cluster(ein); 

    xi: reg iobs_refund age mis_age 
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, cluster(ein); 

    xi: reg iobs_refund lmean_revenue_13 mis_lmean_revenue_13
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, cluster(ein); 

    xi: reg iobs_refund 
            cpa atty other mis_profession
            ltot_wages_13 mis_ltot_wages_13
            self_emp_ind mis_self_emp_ind
            age mis_age
            lmean_revenue_13 mis_lmean_revenue_13
        i.nol_yr 
        lsim_refund_13 lrevenue_13 mis_lrevenue_13
        lassets_13 mis_lassets_13
        lebitda_13 mis_lebitda_13, cluster(ein); 
    #delimit cr

end/*%>*/

* Table A4
capture program drop table_preparers/*%<*/
program define table_preparers

    /*************************************************************************** 
    columns are:
        ein mean_payroll_13 mean_revenue_13 nol_yr prep_ein prep_ssn event_t
        iobs_refund obs_refund_13 sim_refund_13 revenue_13 assets_13 payroll_13
        ebitda_13 zip3 size_cat 
    ****************************************************************************/
    use $analysis/clients.dta, clear
    make_size_bin
    egen prep_id = group(prep_ssn)
    bys prep_id: egen mean_client_size = mean(mean_revenue_13)

    foreach v of varlist revenue_13 assets_13 ebitda_13 payroll_13 {
        destring `v', force replace
    }
    
    * same as size_bin
    #delimit;
    egen prep_size_group = cut(mean_client_size), 
        at(0 462672.9 678111.9 930156.6
           1253742 1701663 2378004 3503668 5842788 
           1.34e7 1.75e7 2.49e7 4.11e7 1.01e8
           3.5e12) icodes;
    #delimit cr
    replace prep_size_group = prep_size_group + 1

    * => external prep is if it starts with X in the EIN or Y in the SSN field,
    * then it will be the firm's own EIN in after the X or Y
    merge m:1 prep_ssn nol_yr using "$analysis/preps", keep(3) nogen

    ****************************************************************************
    * 1. Basic preparer facts: do preparers never claim? is there learning?
    *    1. Share of preps with two or more opportunities that always, that
    *    never, and that sometimes.
    *    2. How do NOL opportunities compare to total number of clients?
    ****************************************************************************
    * Analogous table to Table 4: Tabulation of Preparer Characteristics
    * Average client size
    * Unique preps
    * Average income
    * Average refund size
    * Claim?
    * Average N
    * Always?
    * Never?
    * Aggregate $ and claim $

    gen external_prep = cond(!regexm(prep_ein, "X.*") | !regexm(prep_ssn, "Y.*"), 1, 0)
    keep if external_prep == 1
    gen obs = 1

    #delimit ;
    collapse 
        (sum) prep_count=obs total_claims=iobs_refund client_count=n_clients
        (sum) sim_refund_13 obs_refund_13 
        (mean) n_clients
        (mean) client_refund_size=sim_refund_13 client_revenue=revenue_13 
               client_payroll=payroll_13 client_ebitda=ebitda_13, by(prep_id);
    #delimit cr
    gen always_claim = cond(prep_count > 1 & prep_count < ., ///
                        cond(total_claims == prep_count, 1, 0), .)
    gen never_claim = cond(prep_count > 1 & prep_count < ., ///
                       cond(total_claims == 0, 1, 0), .)

    drop if prep_count < 3 | prep_count == .
    drop if prep_id == .
    xtile count_bin = n_clients, nq(5)
    gen claim_share = total_claims / prep_count
    gen nol_share = prep_count / client_count

    #delimit ;
    tabstat 
        prep_count client_count n_clients 
        nol_share claim_share always_claim never_claim
        client_refund_size client_revenue client_payroll client_ebitda
        , s(mean N);
    tabstat 
        prep_count client_count n_clients 
        nol_share claim_share always_claim never_claim
        client_refund_size client_revenue client_payroll client_ebitda
        , by(count_bin) s(mean N);
    #delimit cr

    use `orig', clear

end/*%>*/

* Table A5
capture program drop table_claimregs/*%<*/
program define table_claimregs

    make_complex
    
    foreach v of varlist sim_refund_13 revenue_13 assets_13 ebitda_13 {
        destring `v', force replace
        gen l`v' = log(`v')
        gen mis_l`v' = l`v' == .
        replace l`v' = -1 if l`v' == .
    }

    xi i.nol_yr

    reg iobs_refund ever_amt ever_credit, cluster(ein)
    reg iobs_refund ever_amt ever_credit _Inol_yr* lsim_refund_13 lrevenue_13 mis_lrevenue_13 lassets_13 mis_lassets_13 lebitda_13 mis_lebitda_13, cluster(ein)

    reg iobs_refund ever_amt ever_credit if size_group == 14, cluster(ein)
    reg iobs_refund ever_amt ever_credit _Inol_yr* if size_group == 14, cluster(ein)
    reg iobs_refund ever_amt ever_credit _Inol_yr* lsim_refund_13 lrevenue_13 mis_lrevenue_13 lassets_13 mis_lassets_13 lebitda_13 mis_lebitda_13 if size_group == 14, cluster(ein)

    reg iobs_refund ever_amt##ever_credit _Inol_yr* lsim_refund_13 lrevenue_13 mis_lrevenue_13 lassets_13 mis_lassets_13 lebitda_13 mis_lebitda_13, cluster(ein)
    reg iobs_refund ever_amt##ever_credit _Inol_yr* lsim_refund_13 lrevenue_13 mis_lrevenue_13 lassets_13 mis_lassets_13 lebitda_13 mis_lebitda_13 if size_group == 14, cluster(ein)

end/*%>*/

*******************************************************************************
* Main
*******************************************************************************
capture program drop main
program define main

	capture log c
	cd $outputdir
	log using "appendix_accountants.log", replace
	 
	*** Tables ***
	
	* Table A2
	table_carryback_stimulus
	
	* Table A4
	table_preparers
	
	* Table A5
	table_claimregs
	
    log close
end

main
