*******************************************************************************
*   Title: make_accountants.do
* Purpose: Builds Stata analysis data sets from CSVs from upstream code
*          available to those with permission to use the data.
*   Owner: Eric Zwick
*    Date: 2020-04-07
*******************************************************************************

capture program drop cpi_deflate_13/*%<*/
program define cpi_deflate_13

    gen `1'_13 = .
    replace `1'_13 = `1'/.673514855 if nol_yr == 1996
    replace `1'_13 = `1'/.688968350 if nol_yr == 1997
    replace `1'_13 = `1'/.699699945 if nol_yr == 1998
    replace `1'_13 = `1'/.715153440 if nol_yr == 1999
    replace `1'_13 = `1'/.739192211 if nol_yr == 2000
    replace `1'_13 = `1'/.760226136 if nol_yr == 2001
    replace `1'_13 = `1'/.772245522 if nol_yr == 2002
    replace `1'_13 = `1'/.789845336 if nol_yr == 2003
    replace `1'_13 = `1'/.810879261 if nol_yr == 2004
    replace `1'_13 = `1'/.838352142 if nol_yr == 2005
    replace `1'_13 = `1'/.865395760 if nol_yr == 2006
    replace `1'_13 = `1'/.890044085 if nol_yr == 2007
    replace `1'_13 = `1'/.924217774 if nol_yr == 2008
    replace `1'_13 = `1'/.920929614 if nol_yr == 2009
    replace `1'_13 = `1'/.936035406 if nol_yr == 2010
    replace `1'_13 = `1'/.965581631 if nol_yr == 2011
    replace `1'_13 = `1'/.985563859 if nol_yr == 2012
    replace `1'_13 = `1'/1 if nol_yr == 2013

end/*%>*/

capture program drop make_size_bin/*%<*/
program define make_size_bin

    /* The ends come from this command.
    egen size_ends = cut(mean_revenue_13), group(50) label
    tab size_ends
    Grab deciles 1-9 and then split 10th into 5 bins
    */

    #delimit ;
    egen size_group = cut(mean_revenue_13), 
        at(0 462672.9 678111.9 930156.6
           1253742 1701663 2378004 3503668 5842788 
           1.34e7 1.75e7 2.49e7 4.11e7 1.01e8
           3.5e12) icodes;
    #delimit cr

    replace size_group = size_group + 1

end/*%>*/

capture program drop build_clients/*%<*/
program define build_clients

    insheet using "$csvdir/clients.csv", clear
    destring mean_revenue_13, force replace
    save "$analysis/clients.dta", replace

end/*%>*/

capture program drop build_switchers/*%<*/
program define build_switchers

    insheet using "$csvdir/switchers.csv", clear
    destring soi_db_ind, force replace
    keep if soi_db_ind == 1
    gen delta_time = event_t - event_t_0
    keep if delta_time >= -1 & delta_time <= 0
    bys ein event_t_0: gen count = _N
    keep if count == 2
    bys ein: egen min_year = min(nol_yr_0)
    keep if nol_yr_0 == min_year
    save "$analysis/switchers.dta", replace

    insheet using "$csvdir/deaths_movers.csv", clear
    destring soi_db_ind, force replace
    keep if soi_db_ind == 1
    gen delta_time = event_t - event_t_0
    keep if delta_time >= -1 & delta_time <= 0
    bys ein event_t_0: gen count = _N
    keep if count == 2
    bys ein: egen min_year = min(nol_yr_0)
    keep if nol_yr_0 == min_year
    save "$analysis/deaths_movers.dta", replace

end/*%>*/

capture program drop build_stimulus/*%<*/
program define build_stimulus

    tempfile sim

    import delimited using $csvdir/refunds_20190913.txt, delim("|") clear
    * Deal with overlap in first 3M to next 3M
    keep if _n <= 3000000
    /* From sim_refunds.py
        outfile.writerow([ein, nol_yr, sim_refund, nol_initial, nol_remaining,
                      sim_refund_2yr, refund_part_2yr[1], refund_part_2yr[2],
                      sim_refund_5yr, refund_part_5yr[1], refund_part_5yr[2],
                      refund_part_5yr[3], refund_part_5yr[4], refund_part_5yr[5]])
    */
    #delimit ;
    rename (v1-v14) (ein nol_yr sim_refund_EZ nol_initial_EZ nol_remaining_EZ
                     sim_refund_2yr sim_refund_2yr_p1 sim_refund_2yr_p2
                     sim_refund_5yr sim_refund_5yr_p1 sim_refund_5yr_p2 
                     sim_refund_5yr_p3 sim_refund_5yr_p4 sim_refund_5yr_p5);
    #delimit cr
    keep if sim_refund_EZ > 0
    * Apply CPI inflation adjustment
    cpi_deflate_13 sim_refund_EZ
    save `sim'

    foreach x in "refunds_3to6M.txt" "refunds_6to9M.txt" "refunds_9to12M.txt" "refunds_12to15M.txt" {

        import delimited using "$csvdir/`x'", delim("|") clear
        #delimit ;
        rename (v1-v14) (ein nol_yr sim_refund_EZ nol_initial_EZ nol_remaining_EZ
                         sim_refund_2yr sim_refund_2yr_p1 sim_refund_2yr_p2
                         sim_refund_5yr sim_refund_5yr_p1 sim_refund_5yr_p2 
                         sim_refund_5yr_p3 sim_refund_5yr_p4 sim_refund_5yr_p5);
        #delimit cr
        keep if sim_refund_EZ > 0
        * Apply CPI inflation adjustment
        cpi_deflate_13 sim_refund_EZ

        append using `sim'
        save `sim', replace

    }

    duplicates drop ein nol_yr, force
    save $analysis/sim_refunds.dta, replace

end/*%>*/

capture program drop build_preparers/*%<*/
program define build_preparers

    insheet using $csvdir/preparers.csv, comma clear

    destring soi_db_ind w2_wages_13 self_emp_incm_13 age mean_revenue_13 n_clients, force replace

    /* Code from nol_takeup_levels.prediction.r */
    * # only tax preparer in SOI DB
    * data = data[soi_db_ind == 1]
    keep if soi_db_ind == 1

    /*
    # wages
    data$tot_wages_13 = data$w2_wages_13 + data$self_emp_incm_13
    data$ltot_wages_13 = log(data$tot_wages_13)
    data$ltot_wages_13 = ifelse(is.na(data$tot_wages_13) | data$tot_wages_13 < 1, 
                                -1, data$ltot_wages_13)
    data$mis_ltot_wages_13 = as.numeric(data$ltot_wages_13 == -1)
    */
    gen tot_wages_13 = w2_wages_13 + self_emp_incm_13
    gen ltot_wages_13 = log(tot_wages_13)
    replace ltot_wages_13 = -1 if tot_wages_13 == . | tot_wages_13 < 1
    gen mis_ltot_wages_13 = ltot_wages_13 == -1

    /*
    # self-employment
    data$self_emp_share = data$self_emp_incm_13 / data$tot_wages_13
    data$self_emp_ind = data$self_emp_share >= 0.5
    data$self_emp_ind = ifelse(is.na(data$self_emp_ind), 
                                -1, data$self_emp_ind)
    data$mis_self_emp_ind = as.numeric(data$self_emp_ind == -1)
    */
    gen self_emp_share = self_emp_incm_13 / tot_wages_13
    gen self_emp_ind = self_emp_share >= 0.5 & self_emp_share != .
    replace self_emp_ind = -1 if self_emp_share == .
    gen mis_self_emp_ind = self_emp_ind == -1

    /*
    # age
    data$age = ifelse(is.na(data$age), 
                -1, data$age)
    data$mis_age = as.numeric(data$age == -1)
    */
    replace age = cond(age != ., age, -1)
    gen mis_age = age == -1

    /*
    # mean clients
    data$lmean_revenue_13 = log(data$mean_revenue_13)
    data$lmean_revenue_13 = ifelse(is.na(data$mean_revenue_13) | data$mean_revenue_13 < 1, 
                                -1, data$lmean_revenue_13)
    data$mis_lmean_revenue_13 = as.numeric(data$lmean_revenue_13 == -1)
    */
    gen lmean_revenue_13 = log(mean_revenue_13)
    replace lmean_revenue_13 = -1 if mean_revenue_13 == . | mean_revenue_13 < 1
    gen mis_lmean_revenue_13 = lmean_revenue_13 == -1

    /*
    # total clients
    data$tot_revenue_13 = data$mean_revenue_13*data$N_clients
    data$ltot_revenue_13 = log(data$tot_revenue_13)
    data$ltot_revenue_13 = ifelse(is.na(data$tot_revenue_13) | data$tot_revenue_13 < 1, 
                                -1, data$ltot_revenue_13)
    data$mis_ltot_revenue_13 = as.numeric(data$ltot_revenue_13 == -1)
    */
    gen tot_revenue_13 = mean_revenue_13 * n_clients
    gen ltot_revenue_13 = log(tot_revenue_13)
    replace ltot_revenue_13 = -1 if tot_revenue_13 == . | tot_revenue_13 < 1
    gen mis_ltot_revenue_13 = ltot_revenue_13 == -1

    /*
    # profession
    data$atty = data$profession=="ATTY"
    data$atty = ifelse(is.na(data$profession), -1, data$atty)
    data$cpa = data$profession=="CPA"
    data$cpa = ifelse(is.na(data$profession), -1, data$cpa)
    data$other = data$profession=="EA" | data$profession=="EAC" | data$profession=="SRTP" 
    data$other = ifelse(is.na(data$profession), -1, data$other)
    data$mis_profession = as.numeric(is.na(data$profession))
    */
    gen atty = profession=="ATTY"
    replace atty = -1 if profession == "NA" | profession == ""
    gen cpa = profession=="CPA"
    replace cpa = -1 if profession == "NA" | profession == ""
    gen other = profession=="EA" | profession=="EAC" | profession=="SRTP" 
    replace other = -1 if profession == "NA" | profession == ""
    gen mis_profession = (profession == "NA" | profession == "")

    #delimit ;
    keep prep_ssn n_clients nol_yr
            cpa atty other mis_profession
            ltot_wages_13 mis_ltot_wages_13
            self_emp_ind mis_self_emp_ind
            age mis_age
            lmean_revenue_13 mis_lmean_revenue_13;
    #delimit cr
    duplicates drop prep_ssn nol_yr, force
    save "$analysis/preps.dta", replace

end/*%>*/

capture program drop build_complex/*%<*/
program define build_complex

    tempfile complex

    * First is the AMT data
    tempfile amt
    insheet using "$csvdir/amt_data.csv", comma clear
    collapse (count) amt_count=tax_yr, by(ein)
    save `amt'

    use $analysis/clients, clear
    merge m:1 ein using `amt', keep(1 3) nogen
    make_size_bin
    gen ever_amt = cond(amt_count != . & amt_count > 0, 1, 0)
    save `complex'

    * Second is the credits data
    tempfile credit
    forval v=1/12 {
        insheet using "$csvdir/complex`v'.csv", comma clear
        collapse (sum) credit_count=tax_credits, by(ein)

        if (`v' == 1) {
            save `credit'
        }
        else {
            append using `credit'
            save `credit', replace
        }
    }
    use `credit', clear
    collapse (sum) credit_count, by(ein)
    save `credit', replace

    use `complex', clear
    merge m:1 ein using `credit', keep(1 3) nogen
    gen ever_credit = cond(credit_count != . & credit_count > 0, 1, 0)

end/*%>*/

*******************************************************************************
* Main
*******************************************************************************
capture program drop main
program define main

	log using "$outputdir/appendix_accountants.log", replace

	build_clients
	build_switchers
	build_stimulus
	build_preparers
	build_complex
	
	log close
end
